

-- Values used often so created variables for them
local addonName = "XrysMapPins"
local addonVersion = "1.0.0"
local addonSV = addonName.."_SavedVariables"
local addonPrefix = "XMP"

---------------------[[ System Managers ]]----------------------
local ChatMsg = CHAT_SYSTEM
local EventMgr = EVENT_MANAGER
local WindowMgr = WINDOW_MANAGER
----------------------------------------------------------------


----------------[[ Saved Variable Data ]]-----------------------
local SVDefault = {}
local SVData = {}

local function InitSavedVariables(...)
    SVData = ZO_SavedVars:NewAccountWide(addonSV, 1, "Data", SVDefault)
end
---------------------------------------------------------------



----------------[[ Tradeskill Information ]]--------------------
local TradeSkillNames = {}
local TradeSkills = {}

local function InitTradeSkillInfo()
	local skillType,skillIndex = GetCraftingSkillLineIndices(CRAFTING_TYPE_ALCHEMY)
	local skillName,skillRank  = GetSkillLineInfo(skillType, skillIndex) 
	local abilityName,abilityIcon = GetSkillAbilityInfo(skillType, skillIndex, 1) 
	TradeSkills[CRAFTING_TYPE_ALCHEMY] = { Name = skillName, Icon = abilityIcon }
  
	skillType,skillIndex = GetCraftingSkillLineIndices(CRAFTING_TYPE_BLACKSMITHING)
	skillName,skillRank  = GetSkillLineInfo(skillType, skillIndex) 
	abilityName,abilityIcon = GetSkillAbilityInfo(skillType, skillIndex, 1) 
	TradeSkills[CRAFTING_TYPE_BLACKSMITHING] = 	{ Name = skillName, Icon = abilityIcon }


	skillType,skillIndex = GetCraftingSkillLineIndices(CRAFTING_TYPE_CLOTHIER)
	skillName,skillRank  = GetSkillLineInfo(skillType, skillIndex) 
	abilityName,abilityIcon = GetSkillAbilityInfo(skillType, skillIndex, 1) 
	TradeSkills[CRAFTING_TYPE_CLOTHIER] = 	{ Name = skillName, Icon = abilityIcon }

	skillType,skillIndex = GetCraftingSkillLineIndices(CRAFTING_TYPE_ENCHANTING)
	skillName,skillRank  = GetSkillLineInfo(skillType, skillIndex) 
	abilityName,abilityIcon = GetSkillAbilityInfo(skillType, skillIndex, 1) 
	TradeSkills[CRAFTING_TYPE_ENCHANTING] = 	{ Name = skillName, Icon = abilityIcon }
  
	skillType,skillIndex = GetCraftingSkillLineIndices(CRAFTING_TYPE_PROVISIONING)
	skillName,skillRank  = GetSkillLineInfo(skillType, skillIndex) 
	abilityName,abilityIcon = GetSkillAbilityInfo(skillType, skillIndex, 1) 
	TradeSkills[CRAFTING_TYPE_PROVISIONING] = 	{ Name = skillName, Icon = abilityIcon }
    
	skillType,skillIndex = GetCraftingSkillLineIndices(CRAFTING_TYPE_WOODWORKING)
	skillName,skillRank  = GetSkillLineInfo(skillType, skillIndex) 
	abilityName,abilityIcon = GetSkillAbilityInfo(skillType, skillIndex, 1) 
	TradeSkills[CRAFTING_TYPE_WOODWORKING] = 	{ Name = skillName, Icon = abilityIcon }

	for i,v in ipairs(TradeSkills) do
		table.insert(TradeSkillNames,v.Name)
	end
end
----------------------------------------------------------------



------------------[[ Map Pin Functionality ]]-------------------
local CustomMapPins = ZO_WorldMapPins:Subclass()

function CustomMapPins:New(...)
	local pinObject = ZO_WorldMapPins.New(self)
	pinObject:Initialise(...)
	return pinObject
end

function CustomMapPins:Initialise(...)

	--Default Values
	self.Type = addonPrefix.."_Unspecified"
	self.Name = "Unspecified"
	self.TradeSkill = CRAFTING_TYPE_INVALID
	self.ItemType = ITEMTYPE_NONE
	self.Icon = "EsoUI/Art/MapPins/hostile_pin.dds"
	self.Size = 16
	self.Level = 20

	-- Useful data
	self.Info = {}
	-- [pinID]
	-- Type
	-- Name	
	-- Icon

	-- Pin Data
	self.Data = {}
	--[pinType][zone]
	-- key
	-- zone
	-- x
	-- y
	-- radius

	-- Layout Data
	self.Layout = { level = self.Level, texture = self.Icon, size = self.Size } 
end

function CustomMapPins:List(...)
	local pinType, pinZone = ...
	ChatMsg:AddMessage("Pin Data currently available for type "..pinType.." in zone "..pinZone)
	if not self.Data[pinType] then return end
	if not self.Data[pinType][pinZone] then return end
	for i,v in pairs(self.Data[pinType][pinZone]) do
		ChatMsg:AddMessage(v.zone.." "..tostring(v.x)..", "..tostring(v.y))
	end
end


function CustomMapPins:CreatePin(...)
	local pinType, pinKey, pinZone, pinX, pinY, pinRadius = ...
	self.Data[pinType] = self.Data[pinType] or {}
	self.Data[pinType][pinZone] = self.Data[pinType][pinZone] or {}
	table.insert(self.Data[pinType][pinZone],{ 
		["key"] = pinKey, 
		["zone"] = pinZone, 
		["x"] = pinX, 
		["y"] = pinY, 
		["radius"] = pinRadius 
	})
end

function CustomMapPins:AddCustomPin(...)
	local pinType, drawCallback, resizeCallback, pinLayout, pinTooltipCreator, pinID, enablePin = ...

	if pinID then
		self.Info[pinID] = self.Info[pinID] or {}
		table.insert(self.Info[pinID], {
			["Type"] = pinType,
			["Name"] = TradeSkills[pinID].Name,
			["Icon"] = TradeSkills[pinID].Icon
			})
	end
	ZO_WorldMap_AddCustomPin(pinType, drawCallback, resizeCallback, pinLayout, pinTooltipCreator)
	self:SetCustomPinEnabled( pinType, enablePin )
	self:RefreshCustomPins(pinType)
end

function CustomMapPins:AddFilter(...)

	local pinType,pinID,enable = ...

	WORLD_MAP_FILTERS.pvePanel:AddPinFilterCheckBox(5, function() self:RefreshCustomPins(pinType) end)		
	WORLD_MAP_FILTERS.pvePanel.lastControl.label:SetText(GetString("SI_MAPFILTER",5).." ("..TradeSkills[pinID].Name..")")
	WORLD_MAP_FILTERS.pvePanel.lastControl:SetState(1)
end

function CustomMapPins:RemovePins(...)
	local pinType = ...
	self.Data[pinType] = {}
	self:RefreshCustomPins(pinType)
end

function CustomMapPins:RefreshCustomPins(pinType)
	ZO_WorldMap_RefreshCustomPinsOfType(_G[pinType])
end

function CustomMapPins:SetCustomPinEnabled(pinType)
	ZO_WorldMap_SetCustomPinEnabled(_G[pinType])
end

function CustomMapPins:IsCustomPinEnabled(pinType)
	return ZO_WorldMap_IsCustomPinEnabled(_G[pinType])
end

function CustomMapPins:IsPinGroupShown(pinType)
	return ZO_WorldMap_IsPinGroupShown(_G[pinType])	
end

-- The following functions are set up as part of ZO_WorldMapPins but can't see a direct call to them
-- to hazard a guess as to whether programs need to customise them

--function CustomMapPins:CreateCustomPinType(...)
--end

--function CustomMapPins:FindPin(...)
--end

--function CustomMapPins:MapPinLookupToPinKey(...)
--end

--function CustomMapPins:UpdatePinsForMapSizeChange(...)
--end


local XMPins = CustomMapPins:New()
--------------------------------------------------------------


------------------------[[ UI Code ]]-------------------------
local function CreateTopLevelWindow(name,hasBackground)
	local tlw = WindowMgr:CreateTopLevelWindow(name)
	tlw:SetMovable(true)
	tlw:SetMouseEnabled(true)
	tlw:SetDimensions(200,100)

	if hasBackground then
		tlw.BackGround = WindowMgr:CreateControlFromVirtual(nil,tlw,"ZO_DefaultBackdrop")
		tlw.BackGround:SetAlpha(0.8)
		tlw.BackGround:SetHidden(false)
	end

	return tlw
end

local function CreateDropDown(name,parent,data,selected,callback)
	local combo = WindowMgr:CreateControlFromVirtual(parent:GetName()..name,parent,"ZO_StatsDropdownRow")
	combo:SetResizeToFitDescendents(false)

	combo.selected = combo.name

	combo.dropdown = combo.dropdown
	combo.dropdown:SetFont("ZoFontBookScroll")
	if selected then combo.dropdown:SetSelectedItem(selected) end

	-- Custom Function To Return the data index which the text came from
	combo.dropdown.GetDataIndexForSelectedItem = function(self,value)
		self.SelectedIndex = nil
		for i = 1,#data do
			if data[i] == value then
				self.SelectedIndex = i
				break
			end
		end
		return self.SelectedIndex
	end

	-- Custom Function to return both the value and the index selected
	combo.GetSelection = function(self)
		return self.dropdown.SelectedItem,self.dropdown.SelectedIndex
	end

	-- Custom Function To Deal with selection of the data
	combo.dropdown.OnSelect = function(self,value)
		self:SetSelectedItem(value)
		self.SelectedItem = value
		local index = self:GetDataIndexForSelectedItem(value)
		if callback then callback(self,value,index) end
	end

	-- Fill Combo Box with Data
	for i = 1,#data do
		local entry = combo.dropdown:CreateItemEntry(data[i],combo.dropdown.OnSelect)
		combo.dropdown:AddItem(entry)
	end

	combo:SetHidden(false)
	return combo
end

local function CreateButton(name,parent,text,callback)
	local button = WindowMgr:CreateControlFromVirtual(parent:GetName()..name,parent,"ZO_DefaultButton")
	button:SetHandler("OnClicked",
		function(self) 
			if callback then callback(self) end
		end)	

	button:SetFont("ZoFontGameLargeBoldShadow")
	button:SetText(text)
	
	return button
end

local function InitUI()
	local testTLW = CreateTopLevelWindow("XrysMapPins_TLW",false)
	testTLW:SetDimensions(100,100)

	local PinTexture = WindowMgr:CreateControl(nil,testTLW,CT_TEXTURE)
	PinTexture:SetDimensions(50,50)
	PinTexture:SetHidden(false)

	-- When you select a tradeskill it will show what icon *should* be used as a map pin texture but a bit bigger
	local MapPinCombo = CreateDropDown("Type",testTLW,TradeSkillNames,nil,function(self,value,index) 
		PinTexture:SetTexture(TradeSkills[index].Icon)
	end)

	-- When you click the button generate a pin at the current location 
	local PutMapPinButton = CreateButton("CreatePin",testTLW,"Create Pin", function(self) 
		local pinType,pinID = MapPinCombo:GetSelection()
		if not pinType or not pinID then ChatMsg:AddMessage(string.format("Pin Type/ID Not Set (pinType:%s) (pinID:%s)",tostring(pinType),tostring(pinID))) return end
		pinType = addonPrefix.."_"..pinType
		SetMapToPlayerLocation()
		local pinZone = GetUnitZone("player")--GetMapName()
		local pinX,pinY,pinHeading = GetMapPlayerPosition("player")		
		local pinTooltip = TradeSkills[pinID].Name
		local pinTag = { ["id"] = pinID, ["type"] = pinType, ["zone"] = pinZone, ["x"] = pinX, ["y"] = pinY, ["tooltip"] = pinTooltip } 
		XMPins:CreatePin(pinType, pinTag, pinZone, pinX, pinY, nil)
		XMPins:RefreshCustomPins(pinType)
		XMPins:List(pinType,pinZone)
	end )
	PutMapPinButton:SetDimensions(200,50)

	-- Position the controls the best we can as that combo box is not consistently sized
	testTLW:SetAnchor(CENTER)
	MapPinCombo:SetAnchor(TOPRIGHT,testTLW,TOPRIGHT,0,0)
	PutMapPinButton:SetAnchor(TOPRIGHT,MapPinCombo,BOTTOMRIGHT,0,0)
	PinTexture:SetAnchor(TOPRIGHT,PutMapPinButton,TOPLEFT,0,0)

    -- Hide the frame
    testTLW:SetHidden(true)

end
------------------------------------------------------------------------

local function InitPinTypes(addFilter)
	for i,v in ipairs(TradeSkills) do
		local pinType = addonPrefix.."_"..v.Name 
		local pinID = i
		local drawCallback = function(pinManager) 

			-- Get the zone the player is in 
			-- this is how the gatherer addon gets it so hopefully it is correct
		 	local pinZone = GetUnitZone("player") --GetMapName()

		 	-- Get the existing pin data for the current type and zone
		 	-- And recreate the pins as we have just redrawn the map
			local harvestData = XMPins.Data[pinType][pinZone]
			for i,v in pairs(harvestData) do
				pinManager:CreatePin(_G[pinType], v.key, v.x, v.y, v.radius)
			end
		end
		local resizeCallback = nil --function(width,height) XMPins:RefreshCustomPins(pinType) end		
		local pinLayoutData = {}
		for i,v in pairs(XMPins.Layout) do
			pinLayoutData[i] = v
		end
		pinLayoutData.texture = TradeSkills[pinID].Icon
		local pinTooltipCreator = { creator = function(pin) InformationTooltip:AddLine(pin.m_PinTag.tooltip) end, tooltip = InformationTooltip }

		-- Add a filter to the built in map filters, it might work (crosses fingers)
		if addFilter then XMPins:AddFilter(pinType,pinID,true) end

		-- Add the custom pin and enable it
		XMPins:AddCustomPin(pinType,drawCallback,resizeCallback,pinLayoutData,pinTooltipCreator,pinID,true) 
	end	
end

-- Process the slash commands
local function ProcessSlashCommands(option)

	-- Extract Options
    local options = {}
    local searchResult = { string.match(option,"^(%S*)%s*(.-)$") }
    for i,v in pairs(searchResult) do
        if (v ~= nil and v ~= "") then
            options[i] = string.lower(v)
        end
    end

    -- Show what slash commands are available
	if #options == 0 or options[1] == "help" then
		ChatMsg:AddMessage("XrysMapPins Slash Commands")
		ChatMsg:AddMessage("/xrmp toggle - toggle the add pin test window")
		return
	end

	-- Execute functionality based on user requests

	-- Refresh the Harvest History table
	if options[1] == "toggle" then
		_G["XrysMapPins_TLW"]:ToggleHidden()
	end

end
-- Initialise slash Commands
local function InitSlashCommands(...)
	SLASH_COMMANDS["/xrmp"] = ProcessSlashCommands
end

local function PlayerActivated(eventID)
	EventMgr:UnregisterForEvent(addonName,eventID)
	ChatMsg:AddMessage(string.format("%s %s Loaded",addonName,addonVersion))
	ChatMsg:AddMessage("/xrmp toggle - toggle the add pin test window")
	InitTradeSkillInfo()
	InitPinTypes(false)
	InitUI()
end

-- Our Addon is loaded so we can start work
local function AddOnLoaded(eventID,addon)
	if addon ~= addonName then return end
	EventMgr:UnregisterForEvent(addonName,eventID)
	InitSavedVariables()
	InitSlashCommands()
	EventMgr:RegisterForEvent( addonName ,EVENT_PLAYER_ACTIVATED , PlayerActivated )
end

-- Track addons loading
EventMgr:RegisterForEvent( addonName ,EVENT_ADD_ON_LOADED , AddOnLoaded )
